#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyams_content.component.theme import ICollectionsTarget, ICollectionsInfo

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.theme.interfaces import ITagsTarget, ITagsInfo, IThemesTarget, IThemesInfo
from pyams_content.shared.view.interfaces import IWfView, IViewQueryEsParamsExtension, IViewThemesSettings, \
    IViewTagsSettings, IViewCollectionsSettings
from pyams_content_es.interfaces import IDocumentIndexInfo
from zope.intid.interfaces import IIntIds

# import packages
from elasticsearch_dsl import Q
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.list import unique
from pyams_utils.registry import get_utility


#
# Tags index
#

@adapter_config(name='tags', context=ITagsTarget, provides=IDocumentIndexInfo)
def tags_target_index_info(content):
    """Tags target index info"""
    intids = get_utility(IIntIds)
    tags = ITagsInfo(content).tags or ()
    return {
        'tags': [intids.register(tag) for tag in unique(tags)]
    }


@adapter_config(name='tags', context=IWfView, provides=IViewQueryEsParamsExtension)
class ViewTagsQueryEsParamsExtension(ContextAdapter):
    """View tags query adapter """

    weight = 60

    def get_es_params(self, context):
        settings = IViewTagsSettings(self.context)
        tags = settings.get_tags_index(context)
        if tags:
            return Q('terms', **{'tags': tags})


#
# Themes index
#

@adapter_config(name='themes', context=IThemesTarget, provides=IDocumentIndexInfo)
def themes_target_index_info(content):
    """Themes target index info"""
    terms = []
    parents = []
    synonyms = []
    associations = []
    intids = get_utility(IIntIds)
    for term in IThemesInfo(content).themes or ():
        terms.append(term)
        if term.usage is not None:
            terms.append(term.usage)
        parents.extend(term.get_parents())
        synonyms.extend(term.used_for)
        associations.extend(term.associations)
    return {
        'themes': {
            'terms': [intids.register(term) for term in unique(terms)],
            'parents': [intids.register(term) for term in unique(parents)],
            'synonyms': [intids.register(term) for term in unique(synonyms)],
            'associations': [intids.register(term) for term in unique(associations)]
        }
    }


@adapter_config(name='themes', context=IWfView, provides=IViewQueryEsParamsExtension)
class ViewThemesQueryEsParamsExtension(ContextAdapter):
    """View themes query adapter """

    weight = 62

    def get_es_params(self, context):
        settings = IViewThemesSettings(self.context)
        themes = settings.get_themes_index(context)
        if themes:
            return Q('terms', **{'themes.terms': themes})


#
# Collections index
#

@adapter_config(name='collections', context=ICollectionsTarget, provides=IDocumentIndexInfo)
def collections_target_index_info(content):
    """Collections target index info"""
    intids = get_utility(IIntIds)
    collections = ICollectionsInfo(content).collections or ()
    return {
        'collections': [intids.register(tag) for tag in unique(collections)]
    }


@adapter_config(name='collections', context=IWfView, provides=IViewQueryEsParamsExtension)
class ViewCollectionsQueryEsParamsExtension(ContextAdapter):
    """View collections query adapter """

    weight = 64

    def get_es_params(self, context):
        settings = IViewCollectionsSettings(self.context)
        collections = settings.get_collections_index(context)
        if collections:
            return Q('terms', **{'collections': collections})
